@@ -1,5 +1,8 @@
+# $Id: Mixed.pm 844 2009-03-05 00:07:26Z rvos $
 package Bio::Phylo::Matrices::Datatype::Mixed;
 use strict;
+use Bio::Phylo::Matrices::Datatype;
+use Bio::Phylo::Util::Exceptions 'throw';
 use vars '@ISA';
 @ISA = qw(Bio::Phylo::Matrices::Datatype);
 
@@ -7,29 +10,28 @@ use vars '@ISA';
 
 =head1 NAME
 
-Bio::Phylo::Matrices::Datatype::Mixed - Datatype subclass,
+Bio::Phylo::Matrices::Datatype::Mixed - Validator subclass,
 no serviceable parts inside
 
 =head1 DESCRIPTION
 
-The Bio::Phylo::Matrices::Datatype::* classes are used to validated data
+The Bio::Phylo::Matrices::Datatype::* classes are used to validate data
 contained by L<Bio::Phylo::Matrices::Matrix> and L<Bio::Phylo::Matrices::Datum>
 objects.
 
 =cut   
 
-    my ( %range, %missing, %gap );
-    my @fields = ( \%range, \%missing, \%gap );
+    my @fields = \( my ( %range, %missing, %gap ) );
     
     sub _new { 
         my ( $package, $self, $ranges ) = @_;
         if ( not UNIVERSAL::isa( $ranges, 'ARRAY' ) ) {
-            die "No ranges specified!";
+            throw 'BadArgs' => "No type ranges specified for 'mixed' data type!"; 
         }
         my $id = $self->get_id;
-        $range{$id}   = [];
-        $missing{$id} = '?';
-        $gap{$id}     = '-';
+        $range{   $id } = [];
+        $missing{ $id } = '?';
+        $gap{     $id } = '-';
         my $start = 0;
         for ( my $i = 0; $i <= ( $#{ $ranges } - 1 ); $i += 2 ) {
             my $type = $ranges->[ $i     ];
@@ -58,6 +60,8 @@ objects.
 
 =item set_missing()
 
+Sets the symbol for missing data.
+
  Type    : Mutator
  Title   : set_missing
  Usage   : $obj->set_missing('?');
@@ -70,12 +74,19 @@ objects.
 
     sub set_missing {
         my ( $self, $missing ) = @_;
-        $missing{ $self->get_id } = $missing;
+        if ( not $missing eq $self->get_gap ) {
+        	$missing{ $self->get_id } = $missing;
+        }
+        else {
+        	throw 'BadArgs' => "Missing character '$missing' already in use as gap character";
+        }        
         return $self;
     }
 
 =item set_gap()
 
+Sets the symbol for gaps.
+
  Type    : Mutator
  Title   : set_gap
  Usage   : $obj->set_gap('-');
@@ -88,7 +99,12 @@ objects.
 
     sub set_gap {
         my ( $self, $gap ) = @_;
-        $gap{ $self->get_id } = $gap;
+        if ( not $gap eq $self->get_missing ) {
+        	$gap{ $self->get_id } = $gap;
+        }
+        else {
+        	throw 'BadArgs' => "Gap character '$gap' already in use as missing character";
+        }
         return $self;
     }
 
@@ -100,6 +116,8 @@ objects.
 
 =item get_missing()
 
+Returns the object's missing data symbol.
+
  Type    : Accessor
  Title   : get_missing
  Usage   : my $missing = $obj->get_missing;
@@ -113,6 +131,8 @@ objects.
 
 =item get_gap()
 
+Returns the object's gap symbol.
+
  Type    : Accessor
  Title   : get_gap
  Usage   : my $gap = $obj->get_gap;
@@ -128,6 +148,8 @@ objects.
 
 =item get_type()
 
+Returns the object's datatype as string.
+
  Type    : Accessor
  Title   : get_type
  Usage   : my $type = $obj->get_type;
@@ -159,14 +181,69 @@ objects.
         return $string;
     }
 
+=item get_type_for_site()
+
+Returns type object for site number.
+
+ Type    : Accessor
+ Title   : get_type_for_site
+ Usage   : my $type = $obj->get_type_for_site(1);
+ Function: Returns data type object for site
+ Returns : A Bio::Phylo::Matrices::Datatype object
+ Args    : None
+
+=cut
+
+    sub get_type_for_site {
+        my ( $self, $i ) = @_;     
+        if ( exists $range{ $self->get_id }->[$i] ) {
+        	return $range{ $self->get_id }->[$i];
+        }
+        else {
+        	return $range{ $self->get_id }->[-1];
+        }
+    }    
+
 =back
 
 =head2 TESTS
 
 =over
 
+=item is_same()
+
+Compares data type objects.
+
+ Type    : Test
+ Title   : is_same
+ Usage   : if ( $obj->is_same($obj1) ) {
+              # do something
+           }
+ Function: Returns true if $obj1 contains the same validation rules
+ Returns : BOOLEAN
+ Args    : A Bio::Phylo::Matrices::Datatype::* object
+
+=cut
+
+	sub is_same {
+		my ( $self, $obj ) = @_;
+		my $id = $self->get_id;
+		return 1 if $id == $obj->get_id;
+		return 0 if $self->get_type    ne $obj->get_type;
+		return 0 if $self->get_gap     ne $obj->get_gap;
+		return 0 if $self->get_missing ne $obj->get_missing;
+		for my $i ( 0 .. $#{ $range{ $self->get_id } } ) {
+			if ( my $subtype = $range{ $self->get_id }->[$i] ) {
+				return 0 if not $subtype->is_same( $obj->get_type_for_site($i) );
+			}
+		}
+		return 1;		
+	}
+
 =item is_valid()
 
+Returns true if argument only contains valid characters
+
  Type    : Test
  Title   : is_valid
  Usage   : if ( $obj->is_valid($datum) ) {
@@ -179,22 +256,38 @@ objects.
 =cut
 
     sub is_valid { 
-        my ( $self, $datum ) = @_;
-        my ( $start, $end ) = ( $datum->get_position - 1, $datum->get_length - 1 );
+        my $self = shift;
+        my $datum = $_[0];
+        my $is_datum_object;
+        my ( $start, $end );
+        if ( UNIVERSAL::can( $datum, 'get_position') and UNIVERSAL::can( $datum, 'get_length' ) ) {
+        	( $start, $end ) = ( $datum->get_position - 1, $datum->get_length - 1 );
+        	$is_datum_object = 1;
+        }
+        else {
+        	$start = 0;
+        	$end = $#_;
+        }     
         my $ranges = $self->$get_ranges;
         my $type;
-        MODEL_RANGE_CHECK: for my $i ( $start .. $end ) {
-            if ( not $type ) {
-                $type = $ranges->[$i];
-            }
-            elsif ( $type != $ranges->[$i] ) {
-                die; # needs to slice
+		MODEL_RANGE_CHECK: for my $i ( $start .. $end ) {
+    		if ( not $type ) {
+        		$type = $ranges->[$i];
             }
+		    elsif ( $type != $ranges->[$i] ) {
+    		    #die; # needs to slice
+    		    return 1; # TODO
+        	}
             else {
-                next MODEL_RANGE_CHECK;
-            }
-        }
-        return $type->is_valid( $datum );
+	            next MODEL_RANGE_CHECK;
+	    	   }
+    	}
+    	if ( $is_datum_object ) {
+        	return $type->is_valid( $datum );
+    	}
+    	else {
+    		return 1; # FIXME
+    	}
     }
     
     sub DESTROY {
@@ -209,6 +302,10 @@ objects.
 
 =back
 
+=cut
+
+# podinherit_insert_token
+
 =head1 SEE ALSO
 
 =over
@@ -221,52 +318,13 @@ objects.
 
 =item L<Bio::Phylo::Manual>
 
-Also see the manual: L<Bio::Phylo::Manual>.
+Also see the manual: L<Bio::Phylo::Manual> and L<http://rutgervos.blogspot.com>.
 
 =back
 
-=head1 FORUM
+=head1 REVISION
 
-CPAN hosts a discussion forum for Bio::Phylo. If you have trouble
-using this module the discussion forum is a good place to start
-posting questions (NOT bug reports, see below):
-L<http://www.cpanforum.com/dist/Bio-Phylo>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<< bug-bio-phylo@rt.cpan.org >>,
-or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Bio-Phylo>. I will be notified,
-and then you'll automatically be notified of progress on your bug as I make
-changes. Be sure to include the following in your request or comment, so that
-I know what version you're using:
-
-$Id: Mixed.pm 3386 2007-03-24 16:22:25Z rvosa $
-
-=head1 AUTHOR
-
-Rutger A. Vos,
-
-=over
-
-=item email: C<< rvosa@sfu.ca >>
-
-=item web page: L<http://www.sfu.ca/~rvosa/>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-The author would like to thank Jason Stajich for many ideas borrowed
-from BioPerl L<http://www.bioperl.org>, and CIPRES
-L<http://www.phylo.org> and FAB* L<http://www.sfu.ca/~fabstar>
-for comments and requests.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2005 Rutger A. Vos, All Rights Reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as Perl
-itself.
+ $Id: Mixed.pm 844 2009-03-05 00:07:26Z rvos $
 
 =cut
 
