@@ -1,24 +1,19 @@
-# $Id: Table.pm 1652 2006-07-13 02:08:23Z rvosa $
-# Subversion: $Rev: 194 $
+# $Id: Table.pm 843 2009-03-04 23:50:27Z rvos $
 package Bio::Phylo::Parsers::Table;
 use strict;
-use Bio::Phylo;
-use Bio::Phylo::Matrices::Matrix;
-use Bio::Phylo::Matrices::Datum;
-use Bio::Phylo::Taxa;
-use Bio::Phylo::Taxa::Taxon;
-
-# One line so MakeMaker sees it.
-use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
+use Bio::Phylo::IO;
+use Bio::Phylo::Factory;
+use vars qw(@ISA);
 
 # classic @ISA manipulation, not using 'base'
-use vars qw($VERSION @ISA);
 @ISA = qw(Bio::Phylo::IO);
 
+my $fac = Bio::Phylo::Factory->new;
+my $logger = Bio::Phylo->get_logger;
+
 =head1 NAME
 
-Bio::Phylo::Parsers::Table - Parses tab- (or otherwise) delimited matrices. No
-serviceable parts inside.
+Bio::Phylo::Parsers::Table - Parser used by Bio::Phylo::IO, no serviceable parts inside
 
 =head1 DESCRIPTION
 
@@ -77,10 +72,10 @@ sub _new {
 sub _from_both {
     my $self   = shift;
     my %opts   = @_;
-    my $matrix = Bio::Phylo::Matrices::Matrix->new(
+    my $matrix = $fac->create_matrix(
         '-type' => $opts{'-type'},
     );
-    my $taxa   = Bio::Phylo::Taxa->new;
+    my $taxa   = $fac->create_taxa;
     $taxa->set_matrix($matrix);
     $matrix->set_taxa($taxa);
     my ( $fieldre, $linere );
@@ -111,13 +106,12 @@ sub _from_both {
         while ( readline( $opts{'-handle'} ) ) {
             chomp;
             my @temp = split( $fieldre, $_ );
-            my $taxon = Bio::Phylo::Taxa::Taxon->new( '-name' => $temp[0], );
+            my $taxon = $fac->create_taxon( '-name' => $temp[0], );
             $taxa->insert($taxon);
-            my $datum = Bio::Phylo::Matrices::Datum->new(
-                '-name' => $temp[0],
-                '-type' => uc $opts{'-type'},
-                '-char' => [ @temp[ 1, -1 ] ],
-                '-pos'  => 0,
+            my $datum = $fac->create_datum(
+                '-name'  => $temp[0],
+                '-type'  => uc $opts{'-type'},
+                '-char'  => [ @temp[ 1, -1 ] ],
             );
             $datum->set_taxon($taxon);
             $taxon->set_data($datum);
@@ -127,22 +121,38 @@ sub _from_both {
     elsif ( $opts{'-string'} ) {
         foreach my $line ( split( $linere, $opts{'-string'} ) ) {
             my @temp = split( $fieldre, $line );
-            my $taxon = Bio::Phylo::Taxa::Taxon->new( '-name' => $temp[0], );
+            my $taxon = $fac->create_taxon( '-name' => $temp[0], );
             $taxa->insert($taxon);
-            my $datum = Bio::Phylo::Matrices::Datum->new(
+            my $datum = $fac->create_datum(
                 '-name' => $temp[0],
                 '-type' => uc $opts{'-type'},
-                '-char' => [ @temp[ 1 .. $#temp ] ],
-                '-pos'  => 0,
+                #'-char' => [ @temp[ 1 .. $#temp ] ],
             );
+            $datum->set_char(@temp[ 1 .. $#temp ]);
             $datum->set_taxon($taxon);
             $taxon->set_data($datum);
             $matrix->insert($datum);
         }
     }
-    return $matrix;
+    
+    if ( $opts{'-project'} ) {
+    	my $taxa = $matrix->make_taxa();
+    	$opts{'-project'}->insert($taxa,$matrix);
+    	return $opts{'-project'};
+    }
+    elsif ( $opts{'-as_project'} ) {
+    	my $proj = $fac->create_project;
+    	my $taxa = $matrix->make_taxa();
+    	$proj->insert($taxa,$matrix);
+    	return $proj;
+    }
+    else {
+    	return $matrix;
+    }
 }
 
+# podinherit_insert_token
+
 =head1 SEE ALSO
 
 =over
@@ -154,52 +164,13 @@ Look there to learn how to parse tab- (or otherwise) delimited matrices.
 
 =item L<Bio::Phylo::Manual>
 
-Also see the manual: L<Bio::Phylo::Manual|Bio::Phylo::Manual>.
+Also see the manual: L<Bio::Phylo::Manual> and L<http://rutgervos.blogspot.com>
 
 =back
 
-=head1 FORUM
+=head1 REVISION
 
-CPAN hosts a discussion forum for Bio::Phylo. If you have trouble
-using this module the discussion forum is a good place to start
-posting questions (NOT bug reports, see below):
-L<http://www.cpanforum.com/dist/Bio-Phylo>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<< bug-bio-phylo@rt.cpan.org >>,
-or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Bio-Phylo>. I will be notified,
-and then you'll automatically be notified of progress on your bug as I make
-changes. Be sure to include the following in your request or comment, so that
-I know what version you're using:
-
-$Id: Table.pm 1652 2006-07-13 02:08:23Z rvosa $
-
-=head1 AUTHOR
-
-Rutger A. Vos,
-
-=over
-
-=item email: C<< rvosa@sfu.ca >>
-
-=item web page: L<http://www.sfu.ca/~rvosa/>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-The author would like to thank Jason Stajich for many ideas borrowed
-from BioPerl L<http://www.bioperl.org>, and CIPRES
-L<http://www.phylo.org> and FAB* L<http://www.sfu.ca/~fabstar>
-for comments and requests.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2005 Rutger A. Vos, All Rights Reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as Perl
-itself.
+ $Id: Table.pm 843 2009-03-04 23:50:27Z rvos $
 
 =cut
 
