@@ -1,44 +1,45 @@
-# $Id: Treedrawer.pm 2196 2006-09-07 21:35:47Z rvosa $
-# Subversion: $Rev: 192 $
+# $Id: Treedrawer.pm 1127 2009-06-16 07:23:13Z rvos $
 package Bio::Phylo::Treedrawer;
 use strict;
-use Bio::Phylo::Forest::Tree;
-use Bio::Phylo::Forest::Node;
-use Bio::Phylo::Util::CONSTANT qw(_TREE_ looks_like_number);
-my @fields = qw(WIDTH HEIGHT MODE SHAPE PADDING NODE_RADIUS TEXT_HORIZ_OFFSET TEXT_VERT_OFFSET TEXT_WIDTH TREE _SCALEX _SCALEY SCALE FORMAT);
+use Bio::Phylo::Util::Logger;
+use Bio::Phylo::Forest::DrawTree;
+use Bio::Phylo::Util::Exceptions 'throw';
+use Bio::Phylo::Util::CONSTANT qw(_TREE_ looks_like_number looks_like_object looks_like_hash looks_like_class);
+my @fields = qw(WIDTH HEIGHT MODE SHAPE PADDING NODE_RADIUS TIP_RADIUS TEXT_HORIZ_OFFSET TEXT_VERT_OFFSET TEXT_WIDTH TREE _SCALEX _SCALEY SCALE FORMAT);
 
-# hashref of available tree drawer modules
-my $drawers = { 'Svg' => 1 };
+my $tips = 0.000_000_000_000_01;
+my $logger = Bio::Phylo::Util::Logger->new;
 
-# One line so MakeMaker sees it.
-use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
 
 =head1 NAME
 
-Bio::Phylo::Treedrawer - An object-oriented facade for drawing trees.
+Bio::Phylo::Treedrawer - Visualizer of tree shapes
 
 =head1 SYNOPSIS
 
+ use Bio::Phylo::IO 'parse';
  use Bio::Phylo::Treedrawer;
- use Bio::Phylo::IO;
- 
+
+ my $string = '((A:1,B:2)n1:3,C:4)n2:0;';
+ my $tree = parse( -format => 'newick', -string => $string )->first;
+
  my $treedrawer = Bio::Phylo::Treedrawer->new(
-    -width  => 400,
+    -width  => 800,
     -height => 600,
     -shape  => 'CURVY', # curvogram
-    -mode   => 'CLADO', # cladogram
+    -mode   => 'PHYLO', # cladogram
     -format => 'SVG'
  );
- 
- my $tree = Bio::Phylo::IO->parse(
-    -format => 'newick',
-    -string => '((A,B),C);'
- )->first;
- 
+
+ $treedrawer->set_scale_options(
+    -width => '100%',
+    -major => '10%', # major cross hatch interval
+    -minor => '2%',  # minor cross hatch interval
+    -label => 'MYA',
+ );
+
  $treedrawer->set_tree($tree);
- $treedrawer->set_padding(50);
- 
- my $string = $treedrawer->draw;
+ print $treedrawer->draw;
 
 =head1 DESCRIPTION
 
@@ -53,6 +54,8 @@ nodes) and calls the appropriate format-specific drawer.
 
 =item new()
 
+Treedrawer constructor.
+
  Type    : Constructor
  Title   : new
  Usage   : my $treedrawer = Bio::Phylo::Treedrawer->new(
@@ -67,35 +70,34 @@ nodes) and calls the appropriate format-specific drawer.
 
 sub new {
     my $class = shift;
-    my $self = {};
-    $self->{'WIDTH'}             = 500;
-    $self->{'HEIGHT'}            = 500;
-    $self->{'MODE'}              = 'CLADO';
-    $self->{'SHAPE'}             = 'RECT';
-    $self->{'PADDING'}           = 50;
-    $self->{'NODE_RADIUS'}       = 1;
-    $self->{'TEXT_HORIZ_OFFSET'} = 6;
-    $self->{'TEXT_VERT_OFFSET'}  = 4;
-    $self->{'TEXT_WIDTH'}        = 150;
-    $self->{'TREE'}              = undef;
-    $self->{'_SCALEX'}           = 1;
-    $self->{'_SCALEY'}           = 1;
-    $self->{'SCALE'}             = {};
-    $self->{'FORMAT'}            = 'SVG';
+    my $self = {
+        'WIDTH'             => 500,
+        'HEIGHT'            => 500,
+        'MODE'              => 'PHYLO',
+        'SHAPE'             => 'CURVY',
+        'PADDING'           => 50,
+        'NODE_RADIUS'       => 1,
+        'TIP_RADIUS'        => 1,
+        'TEXT_HORIZ_OFFSET' => 6,
+        'TEXT_VERT_OFFSET'  => 4,
+        'TEXT_WIDTH'        => 150,
+        'TREE'              => undef,
+        '_SCALEX'           => 1,
+        '_SCALEY'           => 1,
+        'FORMAT'            => 'Svg',
+        'SCALE'             => undef,
+    };
+    bless $self, $class;
+    
     if (@_) {
-        my %opts = @_;
-        foreach my $key ( keys %opts ) {
-            my $localkey = uc $key;
-            $localkey =~ s/-//;
-            unless ( ref $opts{$key} ) {
-                $self->{$localkey} = uc $opts{$key};
-            }
-            else {
-                $self->{$localkey} = $opts{$key}->clone->negative_to_zero;
-            }
+        my %opts = looks_like_hash @_;
+        for my $key ( keys %opts ) {
+            my $mutator = lc $key;
+            $mutator =~ s/^-/set_/;
+            $self->$mutator( $opts{$key} );
         }
     }
-    return bless $self, $class;
+    return $self;
 }
 
 =back
@@ -106,6 +108,8 @@ sub new {
 
 =item set_format()
 
+Sets image format.
+
  Type    : Mutator
  Title   : set_format
  Usage   : $treedrawer->set_format('Svg');
@@ -117,19 +121,21 @@ sub new {
 =cut
 
 sub set_format {
-    my $self = shift;
-    if ( $drawers->{ ucfirst( $_[0] ) } ) {
-        $self->{'FORMAT'} = ucfirst( $_[0] );
+    my ( $self, $format ) = @_;
+    $format = ucfirst( lc( $format ) );
+    if ( looks_like_class __PACKAGE__ . '::' . $format ) {
+        $self->{'FORMAT'} = $format;
+        return $self;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadFormat->throw(
-            error => "\"$_[0]\" is not a valid image format" );
-    }
-    return $self;
+        throw 'BadFormat' => "'$format' is not a valid image format";
+    }    
 }
 
 =item set_width()
 
+Sets image width.
+
  Type    : Mutator
  Title   : set_width
  Usage   : $treedrawer->set_width(1000);
@@ -140,19 +146,20 @@ sub set_format {
 =cut
 
 sub set_width {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'WIDTH'} = $_[0];
+    my ( $self, $width ) = @_;
+    if ( looks_like_number $width && $width > 0 ) {
+        $self->{'WIDTH'} = $width;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$width' is not a valid image width";
     }
     return $self;
 }
 
 =item set_height()
 
+Sets image height.
+
  Type    : Mutator
  Title   : set_height
  Usage   : $treedrawer->set_height(1000);
@@ -163,19 +170,20 @@ sub set_width {
 =cut
 
 sub set_height {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'HEIGHT'} = $_[0];
+    my ( $self, $height ) = @_;
+    if ( looks_like_number $height && $height > 0 ) {
+        $self->{'HEIGHT'} = $height;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$height' is not a valid image height";
     }
     return $self;
 }
 
 =item set_mode()
 
+Sets tree drawing mode.
+
  Type    : Mutator
  Title   : set_mode
  Usage   : $treedrawer->set_mode('clado');
@@ -187,19 +195,20 @@ sub set_height {
 =cut
 
 sub set_mode {
-    my $self = shift;
-    if ( $_[0] =~ m/(clado|phylo)/i ) {
-        $self->{'MODE'} = uc $_[0];
+    my ( $self, $mode ) = @_;
+    if ( $mode =~ m/^(?:clado|phylo)$/i ) {
+        $self->{'MODE'} = uc $mode;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadFormat->throw(
-            error => "\"$_[0]\" is not a valid drawing mode" );
+    	throw 'BadFormat' => "'$mode' is not a valid drawing mode";
     }
     return $self;
 }
 
 =item set_shape()
 
+Sets tree drawing shape.
+
  Type    : Mutator
  Title   : set_shape
  Usage   : $treedrawer->set_shape('rect');
@@ -211,19 +220,20 @@ sub set_mode {
 =cut
 
 sub set_shape {
-    my $self = shift;
-    if ( $_[0] =~ m/(rect|diag|curvy)/i ) {
-        $self->{'SHAPE'} = uc $_[0];
+    my ( $self, $shape ) = @_;
+    if ( $shape =~ m/^(?:rect|diag|curvy)$/i ) {
+        $self->{'SHAPE'} = uc $shape;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadFormat->throw(
-            error => "\"$_[0]\" is not a valid drawing shape" );
+    	throw 'BadFormat' => "'$shape' is not a valid drawing shape";
     }
     return $self;
 }
 
 =item set_padding()
 
+Sets image padding.
+
  Type    : Mutator
  Title   : set_padding
  Usage   : $treedrawer->set_padding(100);
@@ -234,19 +244,20 @@ sub set_shape {
 =cut
 
 sub set_padding {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'PADDING'} = $_[0];
+    my ( $self, $padding ) = @_;
+    if ( looks_like_number $padding && $padding > 0 ) {
+        $self->{'PADDING'} = $padding;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$padding' is not a valid padding value";
     }
     return $self;
 }
 
 =item set_node_radius()
 
+Sets node radius.
+
  Type    : Mutator
  Title   : set_node_radius
  Usage   : $treedrawer->set_node_radius(20);
@@ -257,19 +268,44 @@ sub set_padding {
 =cut
 
 sub set_node_radius {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'NODE_RADIUS'} = $_[0];
+    my ( $self, $radius ) = @_;
+    if ( looks_like_number $radius && $radius >= 0 ) {
+        $self->{'NODE_RADIUS'} = $radius;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$radius' is not a valid node radius value";
+    }
+    return $self;
+}
+
+=item set_tip_radius()
+
+Sets tip radius.
+
+ Type    : Mutator
+ Title   : set_tip_radius
+ Usage   : $treedrawer->set_tip_radius(20);
+ Function: Sets the tip radius in pixels.
+ Returns :
+ Args    : Integer value in pixels.
+
+=cut
+
+sub set_tip_radius {
+    my ( $self, $radius ) = @_;
+    if ( looks_like_number $radius && $radius >= 0 ) {
+        $self->{'TIP_RADIUS'} = $radius;
+    }
+    else {
+    	throw 'BadNumber' => "'$radius' is not a valid tip radius value";
     }
     return $self;
 }
 
 =item set_text_horiz_offset()
 
+Sets text horizontal offset.
+
  Type    : Mutator
  Title   : set_text_horiz_offset
  Usage   : $treedrawer->set_text_horiz_offset(5);
@@ -281,19 +317,20 @@ sub set_node_radius {
 =cut
 
 sub set_text_horiz_offset {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'TEXT_HORIZ_OFFSET'} = $_[0];
+    my ( $self, $offset ) = @_;
+    if ( looks_like_number $offset ) {
+        $self->{'TEXT_HORIZ_OFFSET'} = $offset;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$offset' is not a valid text horizontal offset value";
     }
     return $self;
 }
 
 =item set_text_vert_offset()
 
+Sets text vertical offset.
+
  Type    : Mutator
  Title   : set_text_vert_offset
  Usage   : $treedrawer->set_text_vert_offset(3);
@@ -305,19 +342,20 @@ sub set_text_horiz_offset {
 =cut
 
 sub set_text_vert_offset {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'TEXT_VERT_OFFSET'} = $_[0];
+    my ( $self, $offset ) = @_;
+    if ( looks_like_number $offset ) {
+        $self->{'TEXT_VERT_OFFSET'} = $offset;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$offset' is not a valid text vertical offset value";
     }
     return $self;
 }
 
 =item set_text_width()
 
+Sets text width.
+
  Type    : Mutator
  Title   : set_text_width
  Usage   : $treedrawer->set_text_width(150);
@@ -329,19 +367,20 @@ sub set_text_vert_offset {
 =cut
 
 sub set_text_width {
-    my $self = shift;
-    if ( looks_like_number $_[0] ) {
-        $self->{'TEXT_WIDTH'} = $_[0];
+    my ( $self, $width ) = @_;
+    if ( looks_like_number $width && $width > 0 ) {
+        $self->{'TEXT_WIDTH'} = $width;
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "'$width' is not a valid text width value";
     }
     return $self;
 }
 
 =item set_tree()
 
+Sets tree to draw.
+
  Type    : Mutator
  Title   : set_tree
  Usage   : $treedrawer->set_tree($tree);
@@ -353,19 +392,20 @@ sub set_text_width {
 =cut
 
 sub set_tree {
-    my $self = shift;
-    if ( $_[0]->can('_type') && $_[0]->_type == _TREE_ ) {
-        $self->{'TREE'} = $_[0]->clone->negative_to_zero;
-    }
-    else {
-        Bio::Phylo::Util::Exceptions::ObjectMismatch->throw(
-            error => "\"$_[0]\" is not a valid tree" );
+    my ( $self, $tree ) = @_;
+    if ( looks_like_object $tree, _TREE_ ) {
+    	if ( not $tree->isa('Bio::Phylo::Forest::DrawTree') ) {
+    		$tree = Bio::Phylo::Forest::DrawTree->new( '-tree' => $tree );
+    	}
+        $self->{'TREE'} = $tree->negative_to_zero;
     }
     return $self;
 }
 
 =item set_scale_options()
 
+Sets time scale options.
+
  Type    : Mutator
  Title   : set_scale_options
  Usage   : $treedrawer->set_scale_options(
@@ -388,34 +428,36 @@ sub set_tree {
 
 sub set_scale_options {
     my $self = shift;
-    if ( @_ && !scalar @_ % 2 ) {
-        my %o = @_;    # %options
+    if ( ( @_ && !scalar @_ % 2 ) || ( scalar @_ == 1 && ref $_[0] eq 'HASH' ) ) {
+        my %o; # %options
+        if ( scalar @_ == 1 && ref $_[0] eq 'HASH' ) {
+            %o = %{ $_[0] };
+        }
+        else {
+            %o = looks_like_hash @_;
+        }
         if ( looks_like_number $o{'-width'} or $o{'-width'} =~ m/^\d+%$/ ) {
             $self->{'SCALE'}->{'-width'} = $o{'-width'};
         }
         else {
-            Bio::Phylo::Util::Exceptions::BadArgs->throw(
-                error => "\"$o{'-width'}\" is invalid for '-width'" );
+            throw 'BadArgs' => "\"$o{'-width'}\" is invalid for '-width'";
         }
         if ( looks_like_number $o{'-major'} or $o{'-major'} =~ m/^\d+%$/ ) {
             $self->{'SCALE'}->{'-major'} = $o{'-major'};
         }
         else {
-            Bio::Phylo::Util::Exceptions::BadArgs->throw(
-                error => "\"$o{'-major'}\" is invalid for '-major'" );
+        	throw 'BadArgs' => "\"$o{'-major'}\" is invalid for '-major'";
         }
         if ( looks_like_number $o{'-minor'} or $o{'-minor'} =~ m/^\d+%$/ ) {
             $self->{'SCALE'}->{'-minor'} = $o{'-minor'};
         }
         else {
-            Bio::Phylo::Util::Exceptions::BadArgs->throw(
-                error => "\"$o{'-minor'}\" is invalid for '-minor'" );
+            throw 'BadArgs' => "\"$o{'-minor'}\" is invalid for '-minor'"; 
         }
         $self->{'SCALE'}->{'-label'} = $o{'-label'};
     }
     else {
-        Bio::Phylo::Util::Exceptions::OddHash->throw(
-            error => 'Odd number of elements in hash assignment' );
+        throw 'OddHash' => 'Odd number of elements in hash assignment';
     }
     return $self;
 }
@@ -428,6 +470,8 @@ sub set_scale_options {
 
 =item get_format()
 
+Gets image format.
+
  Type    : Mutator
  Title   : get_format
  Usage   : my $format = $treedrawer->get_format;
@@ -437,12 +481,12 @@ sub set_scale_options {
 
 =cut
 
-sub get_format {
-    return $_[0]->{'FORMAT'};
-}
+sub get_format { shift->{'FORMAT'} }
 
 =item get_width()
 
+Gets image width.
+
  Type    : Mutator
  Title   : get_width
  Usage   : my $width = $treedrawer->get_width;
@@ -452,12 +496,12 @@ sub get_format {
 
 =cut
 
-sub get_width {
-    return $_[0]->{'WIDTH'};
-}
+sub get_width { shift->{'WIDTH'} }
 
 =item get_height()
 
+Gets image height.
+
  Type    : Accessor
  Title   : get_height
  Usage   : my $height = $treedrawer->get_height;
@@ -467,12 +511,12 @@ sub get_width {
 
 =cut
 
-sub get_height {
-    return $_[0]->{'HEIGHT'};
-}
+sub get_height { shift->{'HEIGHT'} }
 
 =item get_mode()
 
+Gets tree drawing mode.
+
  Type    : Accessor
  Title   : get_mode
  Usage   : my $mode = $treedrawer->get_mode('clado');
@@ -482,12 +526,12 @@ sub get_height {
 
 =cut
 
-sub get_mode {
-    return $_[0]->{'MODE'};
-}
+sub get_mode { shift->{'MODE'} }
 
 =item get_shape()
 
+Gets tree drawing shape.
+
  Type    : Accessor
  Title   : get_shape
  Usage   : my $shape = $treedrawer->get_shape;
@@ -498,12 +542,12 @@ sub get_mode {
 
 =cut
 
-sub get_shape {
-    return $_[0]->{'SHAPE'};
-}
+sub get_shape { shift->{'SHAPE'} }
 
 =item get_padding()
 
+Gets image padding.
+
  Type    : Accessor
  Title   : get_padding
  Usage   : my $padding = $treedrawer->get_padding;
@@ -513,12 +557,12 @@ sub get_shape {
 
 =cut
 
-sub get_padding {
-    return $_[0]->{'PADDING'};
-}
+sub get_padding { shift->{'PADDING'} }
 
 =item get_node_radius()
 
+Gets node radius.
+
  Type    : Accessor
  Title   : get_node_radius
  Usage   : my $node_radius = $treedrawer->get_node_radius;
@@ -528,12 +572,27 @@ sub get_padding {
 
 =cut
 
-sub get_node_radius {
-    return $_[0]->{'NODE_RADIUS'};
-}
+sub get_node_radius { shift->{'NODE_RADIUS'} }
+
+=item get_tip_radius()
+
+Gets tip radius.
+
+ Type    : Accessor
+ Title   : get_tip_radius
+ Usage   : my $tip_radius = $treedrawer->get_tip_radius;
+ Function: Gets the tip radius in pixels.
+ Returns : SCALAR
+ Args    : None.
+
+=cut
+
+sub get_tip_radius { shift->{'TIP_RADIUS'} }
 
 =item get_text_horiz_offset()
 
+Gets text horizontal offset.
+
  Type    : Accessor
  Title   : get_text_horiz_offset
  Usage   : my $text_horiz_offset = 
@@ -545,12 +604,12 @@ sub get_node_radius {
 
 =cut
 
-sub get_text_horiz_offset {
-    return $_[0]->{'TEXT_HORIZ_OFFSET'};
-}
+sub get_text_horiz_offset { shift->{'TEXT_HORIZ_OFFSET'} }
 
 =item get_text_vert_offset()
 
+Gets text vertical offset.
+
  Type    : Accessor
  Title   : get_text_vert_offset
  Usage   : my $text_vert_offset = 
@@ -562,12 +621,12 @@ sub get_text_horiz_offset {
 
 =cut
 
-sub get_text_vert_offset {
-    return $_[0]->{'TEXT_VERT_OFFSET'};
-}
+sub get_text_vert_offset { shift->{'TEXT_VERT_OFFSET'} }
 
 =item get_text_width()
 
+Gets text width.
+
  Type    : Accessor
  Title   : get_text_width
  Usage   : my $textwidth = 
@@ -579,12 +638,12 @@ sub get_text_vert_offset {
 
 =cut
 
-sub get_text_width {
-    return $_[0]->{'TEXT_WIDTH'};
-}
+sub get_text_width { shift->{'TEXT_WIDTH'} }
 
 =item get_tree()
 
+Gets tree to draw.
+
  Type    : Accessor
  Title   : get_tree
  Usage   : my $tree = $treedrawer->get_tree;
@@ -595,12 +654,12 @@ sub get_text_width {
 
 =cut
 
-sub get_tree {
-    return $_[0]->{'TREE'};
-}
+sub get_tree { shift->{'TREE'} }
 
 =item get_scale_options()
 
+Gets time scale option.
+
  Type    : Accessor
  Title   : get_scale_options
  Usage   : my %options = %{ 
@@ -613,9 +672,7 @@ sub get_tree {
 
 =cut
 
-sub get_scale_options {
-    return $_[0]->{'SCALE'};
-}
+sub get_scale_options { shift->{'SCALE'} }
 
 =begin comment
 
@@ -636,15 +693,12 @@ sub _set_scalex {
         $self->{'_SCALEX'} = $_[0];
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid number value" );
+    	throw 'BadNumber' => "\"$_[0]\" is not a valid number value";
     }
     return $self;
 }
 
-sub _get_scalex {
-    return $_[0]->{'_SCALEX'};
-}
+sub _get_scalex { shift->{'_SCALEX'} }
 
 =begin comment
 
@@ -665,15 +719,12 @@ sub _set_scaley {
         $self->{'_SCALEY'} = $_[0];
     }
     else {
-        Bio::Phylo::Util::Exceptions::BadNumber->throw(
-            error => "\"$_[0]\" is not a valid integer value" );
+    	throw 'BadNumber' => "\"$_[0]\" is not a valid integer value";
     }
     return $self;
 }
 
-sub _get_scaley {
-    return $_[0]->{'_SCALEY'};
-}
+sub _get_scaley { shift->{'_SCALEY'} }
 
 =back
 
@@ -683,6 +734,8 @@ sub _get_scaley {
 
 =item draw()
 
+Creates tree drawing. Requires L<SVG>;
+
  Type    : Unparsers
  Title   : draw
  Usage   : my $drawing = $treedrawer->draw;
@@ -690,48 +743,122 @@ sub _get_scaley {
            object into a drawing.
  Returns : SCALAR
  Args    :
+ Notes   : This will only work if you have the SVG module
+           from CPAN installed on your system.
 
 =cut
 
 sub draw {
     my $self = shift;
     if ( !$self->get_tree ) {
-        Bio::Phylo::Util::Exceptions::BadArgs->throw(
-            error => "Can't draw an undefined tree" );
+        throw 'BadArgs' => "Can't draw an undefined tree";
     }
     my $root = $self->get_tree->get_root;
+
+    #Reset the stored data in the tree
+    $self->_reset_internal($root);    
+        
+    if ( $self->get_mode eq 'CLADO' ) {
+        $self->_compute_rooted_clado_coordinates;
+    }
+    elsif ( $self->get_mode eq 'PHYLO' ) {
+        $self->_compute_rooted_phylo_coordinates;
+    }    
+    
+    return $self->render;
+}
+
+sub _compute_rooted_clado_coordinates {
+	my $self = shift;
+	my $root = $self->get_tree->get_root;
     my $tips = $self->get_tree->calc_number_of_terminals;
     my ( $width,   $height )    = ( $self->get_width,   $self->get_height );
     my ( $padding, $textwidth ) = ( $self->get_padding, $self->get_text_width );
-    my $maxpath;
-    if ( $self->get_mode eq 'CLADO' ) {
-        $maxpath = $root->calc_max_nodes_to_tips;
-    }
-    elsif ( $self->get_mode eq 'PHYLO' ) {
-        $maxpath = $root->calc_max_path_to_tips;
+    my $maxpath = $root->calc_max_nodes_to_tips;
+    $self->_set_scalex(
+        ( ( $width - ( ( 2 * $padding ) + $textwidth ) ) / $maxpath ) );
+    $self->_set_scaley( ( ( $height - ( 2 * $padding ) ) / ( $tips + 1 ) ) );
+    $self->_x_positions_clado;    
+    $self->_y_terminals(0);
+    $self->_y_terminals($root);
+    $self->get_tree->get_root->set_y(0);
+    $self->_y_internals;	
+}
+
+sub _compute_rooted_phylo_coordinates {
+	my $self = shift;
+	my $root = $self->get_tree->get_root;
+    my $tips = $self->get_tree->calc_number_of_terminals;
+    my ( $width,   $height )    = ( $self->get_width,   $self->get_height );
+    my ( $padding, $textwidth ) = ( $self->get_padding, $self->get_text_width );
+    my $maxpath = $root->calc_max_path_to_tips;
+    if ( not $maxpath ) {
+        $logger->warn("no branch lengths on tree, switching to clado mode");
+        $self->_compute_clado_coordinates;
+        return;
     }
     $self->_set_scalex(
         ( ( $width - ( ( 2 * $padding ) + $textwidth ) ) / $maxpath ) );
     $self->_set_scaley( ( ( $height - ( 2 * $padding ) ) / ( $tips + 1 ) ) );
-    if ( $self->get_mode eq 'CLADO' ) {
-        $self->_x_positions_clado;
-    }
-    else {
-        $self->_x_positions;
-    }
+    $self->_x_positions_phylo;    
+    $self->_y_terminals(0);
     $self->_y_terminals($root);
-    $self->_y_internals;
-    my $library = __PACKAGE__ . '::' . ucfirst( $self->get_format );
-    eval "require $library";
-    if ($@) {
-        Bio::Phylo::Util::Exceptions::BadFormat->throw(
-            error => "Can't load image drawer: $@" );
-    }
+    $self->get_tree->get_root->set_y(0);
+    $self->_y_internals;	
+}
+
+=item render()
+
+Renders tree based on pre-computed node coordinates. You would typically use
+this method if you have passed a Bio::Phylo::Forest::DrawTree on which you
+have already calculated the node coordinates separately.
+
+ Type    : Unparsers
+ Title   : render
+ Usage   : my $drawing = $treedrawer->render;
+ Function: Unparses a Bio::Phylo::Forest::DrawTree 
+           object into a drawing.
+ Returns : SCALAR
+ Args    :
+ Notes   : This will only work if you have the SVG module
+           from CPAN installed on your system.
+
+=cut
+
+sub render {
+	my $self = shift;
+    my $library = looks_like_class __PACKAGE__ . '::' . ucfirst( lc( $self->get_format ) );
     my $drawer = $library->_new(
-        -tree   => $self->get_tree,
-        -drawer => $self
+        '-tree'   => $self->get_tree,
+        '-drawer' => $self
     );
-    $drawer->_draw;
+    return $drawer->_draw;	
+}
+
+=begin comment
+
+ Type    : Internal method.
+ Title   : _reset_internal
+ Usage   : $treedrawer->_reset_internal;
+ Function: resets the set_generic values stored by Treedrawer, this must be 
+           called at the start of each draw command or weird results are obtained!
+ Returns : nothing
+ Args    : treedrawer, node being processed
+
+=end comment
+
+=cut
+
+sub _reset_internal {
+    my ($self, $node) = @_;
+    my $tree = $self->get_tree;
+    $node->set_x(undef);
+    $node->set_y(undef);
+    my $children = $node->get_children;
+    foreach $node (@$children) {
+        _reset_internal($self,$node);
+    }
+
 }
 
 =begin comment
@@ -747,7 +874,7 @@ sub draw {
 
 =cut
 
-sub _x_positions {
+sub _x_positions_phylo {
     my $self    = shift;
     my $tree    = $self->get_tree;
     my $root    = $tree->get_root;
@@ -755,7 +882,7 @@ sub _x_positions {
     my $padding = $self->get_padding;
     foreach my $node ( @{ $tree->get_entities } ) {
         my $x = ( $node->calc_path_to_root * $scalex ) + $padding;
-        $node->set_generic( x => $x );
+        $node->set_x( $x );
     }
 }
 
@@ -778,28 +905,27 @@ sub _x_positions_clado {
     my $root    = $tree->get_root;
     my $longest = $root->calc_max_nodes_to_tips;
     my $scalex  = $self->_get_scalex;
-    foreach my $tip ( @{ $tree->get_terminals } ) {
-        $tip->set_generic( x => ( ( $longest - 1 ) * $scalex ) );
+    my $padding = $self->get_padding;
+    for my $tip ( @{ $tree->get_terminals } ) {
+        $tip->set_x( $longest * $scalex );
     }
-    foreach my $node ( @{ $tree->get_internals } ) {
+    for my $internal ( @{ $tree->get_internals } ) {
+        my $id = $internal->get_id;
         my $longest1 = 0;
-        foreach my $f ( @{ $tree->get_entities } ) {
-            my ( $q, $current1 ) = ( $f, 0 );
-            if ( !$q->get_first_daughter && $q->get_parent ) {
-                while ($q) {
+        for my $node ( @{ $tree->get_entities } ) {
+            my ( $n, $current1 ) = ( $node, 0 );
+            if ( $n->is_terminal && $n->get_parent ) {
+                while ( $n->get_parent ) {
                     $current1++;
-                    $q = $q->get_parent;
-                    if (   $q
-                        && $q == $node
-                        && $current1 > $longest1 )
-                    {
+                    $n = $n->get_parent;
+                    if ( $n->get_id == $id && $current1 > $longest1 ) {
                         $longest1 = $current1;
                     }
                 }
             }
         }
-        my $xc = $longest - $longest1 - 1;
-        $node->set_generic( x => ( $xc * $scalex ) );
+        my $xc = $longest - $longest1;
+        $internal->set_x( ( $xc * $scalex ) + $padding );
     }
 }
 
@@ -817,15 +943,13 @@ sub _x_positions_clado {
 =cut
 
 {
-    my $tips = 0.000_000_000_000_01;
 
     sub _y_terminals {
-        my $self = shift;
-        my $node = $_[0];
+        my ($self, $node) = @_;
+        if ($node == 0) { $tips = 0.000_000_000_000_01; return; }
         if ( !$node->get_first_daughter ) {
             $tips++;
-            $node->set_generic(
-                y => ( ( $tips * $self->_get_scaley ) + $self->get_padding ) );
+            $node->set_y( ( $tips * $self->_get_scaley ) + $self->get_padding );
         }
         else {
             $node = $node->get_first_daughter;
@@ -854,13 +978,13 @@ sub _x_positions_clado {
 sub _y_internals {
     my $self = shift;
     my $tree = $self->get_tree;
-    while ( !$tree->get_root->get_generic('y') ) {
+    while ( !$tree->get_root->get_y ) {
         foreach my $e ( @{ $tree->get_internals } ) {
-            my $y1 = $e->get_first_daughter->get_generic('y');
-            my $y2 = $e->get_last_daughter->get_generic('y');
+            my $y1 = $e->get_first_daughter->get_y;
+            my $y2 = $e->get_last_daughter->get_y;
             if ( $y1 && $y2 ) {
                 my $y = ( $y1 + $y2 ) / 2;
-                $e->set_generic( 'y' => $y );
+                $e->set_y( $y );
             }
         }
     }
@@ -879,52 +1003,13 @@ Look there for more methods applicable to the treedrawer object.
 
 =item L<Bio::Phylo::Manual>
 
-Also see the manual: L<Bio::Phylo::Manual>.
+Also see the manual: L<Bio::Phylo::Manual> and L<http://rutgervos.blogspot.com>.
 
 =back
 
-=head1 FORUM
+=head1 REVISION
 
-CPAN hosts a discussion forum for Bio::Phylo. If you have trouble
-using this module the discussion forum is a good place to start
-posting questions (NOT bug reports, see below):
-L<http://www.cpanforum.com/dist/Bio-Phylo>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<< bug-bio-phylo@rt.cpan.org >>,
-or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Bio-Phylo>. I will be notified,
-and then you'll automatically be notified of progress on your bug as I make
-changes. Be sure to include the following in your request or comment, so that
-I know what version you're using:
-
-$Id: Treedrawer.pm 2196 2006-09-07 21:35:47Z rvosa $
-
-=head1 AUTHOR
-
-Rutger Vos,
-
-=over
-
-=item email: C<< rvosa@sfu.ca >>
-
-=item web page: L<http://www.sfu.ca/~rvosa/>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-The author would like to thank Jason Stajich for many ideas borrowed
-from BioPerl L<http://www.bioperl.org>, and CIPRES
-L<http://www.phylo.org> and FAB* L<http://www.sfu.ca/~fabstar>
-for comments and requests.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2005 Rutger Vos, All Rights Reserved.
-This program is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+ $Id: Treedrawer.pm 1127 2009-06-16 07:23:13Z rvos $
 
 =cut
 
