@@ -1,19 +1,13 @@
-# $Id: Mrp.pm 2108 2006-08-29 20:46:17Z rvosa $
-# Subversion: $Rev: 190 $
+# $Id: Mrp.pm 843 2009-03-04 23:50:27Z rvos $
 package Bio::Phylo::Unparsers::Mrp;
 use strict;
 use Bio::Phylo::IO;
-
-use vars '@ISA';
+use vars qw(@ISA);
 @ISA=qw(Bio::Phylo::IO);
 
-# One line so MakeMaker sees it.
-use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
-
 =head1 NAME
 
-Bio::Phylo::Unparsers::Mrp - Unparses a forest object into an MRP matrix. No
-serviceable parts inside.
+Bio::Phylo::Unparsers::Mrp - Serializer used by Bio::Phylo::IO, no serviceable parts inside
 
 =head1 DESCRIPTION
 
@@ -128,63 +122,26 @@ sub _to_string {
     return $string;
 }
 
+# podinherit_insert_token
+
 =head1 SEE ALSO
 
 =over
 
 =item L<Bio::Phylo::IO>
 
-The newick unparser is called by the L<Bio::Phylo::IO|Bio::Phylo::IO> object.
-Look there to learn how to unparse newick strings.
+The newick unparser is called by the L<Bio::Phylo::IO> object.
+Look there to learn how to create mrp matrices.
 
 =item L<Bio::Phylo::Manual>
 
-Also see the manual: L<Bio::Phylo::Manual|Bio::Phylo::Manual>.
+Also see the manual: L<Bio::Phylo::Manual> and L<http://rutgervos.blogspot.com>.
 
 =back
 
-=head1 FORUM
+=head1 REVISION
 
-CPAN hosts a discussion forum for Bio::Phylo. If you have trouble
-using this module the discussion forum is a good place to start
-posting questions (NOT bug reports, see below):
-L<http://www.cpanforum.com/dist/Bio-Phylo>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<< bug-bio-phylo@rt.cpan.org >>,
-or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Bio-Phylo>. I will be notified,
-and then you'll automatically be notified of progress on your bug as I make
-changes. Be sure to include the following in your request or comment, so that
-I know what version you're using:
-
-$Id: Mrp.pm 2108 2006-08-29 20:46:17Z rvosa $
-
-=head1 AUTHOR
-
-Rutger A. Vos,
-
-=over
-
-=item email: C<< rvosa@sfu.ca >>
-
-=item web page: L<http://www.sfu.ca/~rvosa/>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-The author would like to thank Jason Stajich for many ideas borrowed
-from BioPerl L<http://www.bioperl.org>, and CIPRES
-L<http://www.phylo.org> and FAB* L<http://www.sfu.ca/~fabstar>
-for comments and requests.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2005 Rutger A. Vos, All Rights Reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as Perl
-itself.
+ $Id: Mrp.pm 843 2009-03-04 23:50:27Z rvos $
 
 =cut
 
