@@ -1,20 +1,15 @@
-# $Id: Pagel.pm 3387 2007-03-25 16:06:50Z rvosa $
-# Subversion: $Rev: 191 $
+# $Id: Pagel.pm 843 2009-03-04 23:50:27Z rvos $
 package Bio::Phylo::Unparsers::Pagel;
 use strict;
 use Bio::Phylo::Forest::Tree;
 use Bio::Phylo::IO;
+use vars qw(@ISA);
 
-use vars '@ISA';
 @ISA=qw(Bio::Phylo::IO);
 
-# One line so MakeMaker sees it.
-use Bio::Phylo; our $VERSION = $Bio::Phylo::VERSION;
-
 =head1 NAME
 
-Bio::Phylo::Unparsers::Pagel - Unparses pagel data files. No serviceable parts
-inside.
+Bio::Phylo::Unparsers::Pagel - Serializer used by Bio::Phylo::IO, no serviceable parts inside
 
 =head1 DESCRIPTION
 
@@ -34,11 +29,26 @@ offspring name, parent name, branch length, character state(s).
 
 =back
 
+Here is an example of what the output might look like:
+
+ 4 2
+ A,n1,0.000000,1,2
+ B,n1,0.000000,1,2
+ n1,n2,0.000000
+ C,n2,0.000000,2,2
+ n2,n3,0.000000
+ D,n3,0.000000,2,1
+
+To the unparse() function pass a tree object as value of the '-phylo'
+argument. The tips in this tree must be linked to taxon objects, and
+the taxon objects must be linked to datum objects whose character
+state sequences are to be serialized.
+
 During unparsing, the tree is randomly resolved, and branch lengths are
 formatted to %f floats (i.e. integers, decimal point, integers).
 
-The pagel module is called by the L<Bio::Phylo::IO|Bio::Phylo::IO> object, so
-look there to learn how to create Pagel formatted files.
+The pagel module is called by the L<Bio::Phylo::IO> object, so
+look there to learn about parsing and serializing in general.
 
 =begin comment
 
@@ -121,63 +131,26 @@ sub _to_string {
     return $string;
 }
 
+# podinherit_insert_token
+
 =head1 SEE ALSO
 
 =over
 
 =item L<Bio::Phylo::IO>
 
-The pagel unparser is called by the L<Bio::Phylo::IO|Bio::Phylo::IO> object.
+The pagel unparser is called by the L<Bio::Phylo::IO> object.
 Look there to learn how to create pagel formatted files.
 
 =item L<Bio::Phylo::Manual>
 
-Also see the manual: L<Bio::Phylo::Manual|Bio::Phylo::Manual>.
+Also see the manual: L<Bio::Phylo::Manual> and L<http://rutgervos.blogspot.com>.
 
 =back
 
-=head1 FORUM
+=head1 REVISION
 
-CPAN hosts a discussion forum for Bio::Phylo. If you have trouble
-using this module the discussion forum is a good place to start
-posting questions (NOT bug reports, see below):
-L<http://www.cpanforum.com/dist/Bio-Phylo>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<< bug-bio-phylo@rt.cpan.org >>,
-or through the web interface at
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Bio-Phylo>. I will be notified,
-and then you'll automatically be notified of progress on your bug as I make
-changes. Be sure to include the following in your request or comment, so that
-I know what version you're using:
-
-$Id: Pagel.pm 3387 2007-03-25 16:06:50Z rvosa $
-
-=head1 AUTHOR
-
-Rutger A. Vos,
-
-=over
-
-=item email: C<< rvosa@sfu.ca >>
-
-=item web page: L<http://www.sfu.ca/~rvosa/>
-
-=back
-
-=head1 ACKNOWLEDGEMENTS
-
-The author would like to thank Jason Stajich for many ideas borrowed
-from BioPerl L<http://www.bioperl.org>, and CIPRES
-L<http://www.phylo.org> and FAB* L<http://www.sfu.ca/~fabstar>
-for comments and requests.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2005 Rutger A. Vos, All Rights Reserved. This program is free
-software; you can redistribute it and/or modify it under the same terms as Perl
-itself.
+ $Id: Pagel.pm 843 2009-03-04 23:50:27Z rvos $
 
 =cut
 
